package com.smppw.analysis.application.service.info;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.application.dto.info.*;
import com.smppw.analysis.domain.dataobject.FundArchivesInfoDO;
import com.smppw.analysis.domain.dataobject.ManualFundNoticeInfoDO;
import com.smppw.analysis.domain.dataobject.FundSimilarDo;
import com.smppw.analysis.domain.dto.info.FundSimilarParams;
import com.smppw.analysis.domain.dto.info.FundSimilarVO;
import com.smppw.analysis.domain.dto.info.ManualFundInvestInfoVO;
import com.smppw.analysis.domain.dto.info.ManualFundNoticeInfoVO;
import com.smppw.analysis.domain.dto.performance.IndicatorParams;
import com.smppw.analysis.domain.manager.performance.Performance;
import com.smppw.analysis.domain.manager.performance.PerformanceConstants;
import com.smppw.analysis.domain.manager.performance.PerformanceFactory;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.analysis.domain.service.NavService;
import com.smppw.common.pojo.IStrategy;
import com.smppw.common.pojo.NewSubStrategy;
import com.smppw.common.pojo.dto.DateValue;
import com.smppw.common.pojo.dto.calc.IndicatorCalcPropertyDto;
import com.smppw.common.pojo.dto.calc.IndicatorCalcTimeRangeDto;
import com.smppw.common.pojo.enums.*;
import com.smppw.core.IndicatorService;
import com.smppw.utils.CalcUtils;
import com.smppw.utils.NavUtil;
import com.smppw.utils.StrategyHandleUtils;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.stream.Collectors;

@Service
public class FundInfoService {
    private final NavService navService;
    private final PerformanceFactory factory;
    private final BaseInfoService baseInfoService;
    private final BaseIndicatorServiceV2 baseIndicatorServiceV2;

    public FundInfoService(NavService navService, PerformanceFactory factory,
                           BaseInfoService baseInfoService, BaseIndicatorServiceV2 baseIndicatorServiceV2) {
        this.factory = factory;
        this.navService = navService;
        this.baseInfoService = baseInfoService;
        this.baseIndicatorServiceV2 = baseIndicatorServiceV2;
    }

    public PrivatelyFundHeadInfoVO headInfo(HeadInfoReq params) {
        return new PrivatelyFundHeadInfoVO();
    }

    public CommonInfoVO commonInfo(CommonInfoReq params) {
//        CommonInfoVO res = new CommonInfoVO();
//        IStrategy strategy = StrategyHandleUtils.getStrategy(params.getStrategy());
//        String secId = params.getSecId();
//        Byte raiseType = null;
//        List<Frequency> result = CollectionUtil.newArrayList();
//        String fundType = this.baseInfoService.getFundType(secId);
//        List<IStrategy> originStrategy = CollectionUtil.newArrayList();
//        // 获取频率
//        CmBigDataFundInfoDo fundHeadInfo = fundHeadInfoBaseService.getFundHeadInfo(secId);
//        if (fundHeadInfo == null) {
//            throw new APIException("");
//        }
//        raiseType = fundHeadInfo.getRaiseType();
//        Frequency navFrequency = fundInformationDao.getNavFrequency(secId);
//        result.add(navFrequency);
//        // 将频率转换一道 取最小频率
//        List<Frequency> frequencies = fundInformationDao.toFrequencyList(result);
//        if (raiseType == 2) {
//            frequencies.removeIf(p -> p.getId() == Frequency.Default.getId());
//        }
//        List<ValueLabelVO> frequencyValueLabel = trans2ValueLabel(frequencies);
//        res.setFrequency(frequencyValueLabel);
//        // 净值类型
//        List<ValueLabelVO> navType = Arrays.stream(NavType.values())
//                .filter(p -> !NavType.All.equals(p) && !NavType.UnitAndCumulativeNav.equals(p))
//                .map(p -> new ValueLabelVO(p.name(), p.getDesc())).collect(Collectors.toList());
//        res.setNavType(navType);
//        // 策略曲线
//        RaiseType realRaiseType = RaiseType.getRaiseType(raiseType);
//        handleTimeRange(strategy, res, secId, companyOrManager, realRaiseType);
//        // 获取常用指数
////        List<ValueLabelVO> benchmarkList = userCommonIndexService.getByUser(-1);
////        handleBenchmark(strategy, res, secId, fundType, originStrategy, companyOrManager, benchmarkList);
        return new CommonInfoVO();
    }

    @SuppressWarnings("unchecked")
    public HeadIndicatorVO headIndicator(HeadIndicatorReq req) {
        IndicatorParams params = req.convert();
        Performance<IndicatorParams, Map<String, Object>> instance = this.factory.getInstance(PerformanceConstants.INDICATOR);
        Map<String, Object> dataset = instance.execute(params);
        Map<String, Object> indicatorMap = MapUtil.get(dataset, req.getSecId(), Map.class);
        HeadIndicatorVO vo = new HeadIndicatorVO();
        vo.setAnnualReturn(MapUtil.getStr(indicatorMap, Indicator.AnnualReturn.name()));
        vo.setAnnualStdDev(MapUtil.getStr(indicatorMap, Indicator.AnnualStdDev.name()));
        vo.setMaxDrawdown(MapUtil.getStr(indicatorMap, Indicator.MaxDrawdown.name()));
        vo.setSharpeRatio(MapUtil.getStr(indicatorMap, Indicator.SharpeRatio.name()));
        return vo;
    }

    /**
     * 私募基金相似产品，计算相关性时代码可以优化（参考基金相关性）
     *
     * @param req /
     * @return /
     */
    public List<FundSimilarVO> getFundSimilarList(FundSimilarReq req) {
        List<String> refIds = ListUtil.toLinkedList(req.getRefId());
        IStrategy strategy = StrategyHandleUtils.getStrategy(req.getStrategy());
        Map<String, List<DateValue>> secNavMap = this.navService.getSecIdDateValueNavListMapByDb(refIds,
                null, null, Visibility.Both, NavType.CumulativeNav, MapUtil.empty());
        List<IndicatorCalcTimeRangeDto> secTimeRanges = IndicatorService.getInstance().getSecTimeRange(req.getRefId(), Frequency.Monthly, secNavMap);
        String startDate = null;
        String endDate = null;
        if (CollUtil.isNotEmpty(secTimeRanges)) {
            for (IndicatorCalcTimeRangeDto secTimeRange : secTimeRanges) {
                if (TimeRange.Last1Year == secTimeRange.getTimeRange()) {
                    startDate = secTimeRange.getStartDate();
                    endDate = secTimeRange.getEndDate();
                    break;
                }
            }
        }
        FundSimilarParams params = req.convert();
        params.setStartDate(startDate);
        List<FundSimilarDo> dataList = this.baseInfoService.getFundSimilarList(params);
        List<String> fundIds = dataList.stream().map(FundSimilarDo::getFundId).collect(Collectors.toList());
        if (CollUtil.isNotEmpty(fundIds)) {
            refIds.addAll(fundIds);
        }
        Map<String, List<DateValue>> allNavMap = this.navService.getSecIdDateValueNavListMapByDb(refIds,
                startDate, endDate, Visibility.Both, NavType.CumulativeNav, MapUtil.empty());
        Map<String, List<IndicatorCalcPropertyDto>> multiSecRetListNew = this.baseIndicatorServiceV2.getMultiSecRetListNew(
                refIds, ListUtil.empty(), Frequency.Monthly, Frequency.Daily, startDate, endDate, false, params.getBenchmarkId(),
                params.getRaiseType(), strategy, Visibility.Both, NavType.CumulativeNav, allNavMap);
        List<Map<String, Object>> initRetList = NavUtil.handleNextStepNew(refIds, ListUtil.empty(), multiSecRetListNew, false);
        Map<String, Map<String, Double>> retMap = this.transfer2TwoDimensionMap(initRetList);
        Double[][] corArr = CalcUtils.getCorrelationMatrix(retMap, ListUtil.toList(params.getRefId()), refIds);
        List<FundSimilarVO> resultList = ListUtil.list(true);
        for (FundSimilarDo map : dataList) {
            FundSimilarVO similarVO = BeanUtil.copyProperties(map, FundSimilarVO.class);
            // 因为第一个基金是当前基金，所以需要 + 1 来取该基金与当前基金的相关性
            int i = fundIds.indexOf(map.getFundId()) + 1;
            similarVO.setSimilar(corArr[0][i]);
            similarVO.setStrategy(StrategyHandleUtils.getStrategy(map.getStrategy()).getStrategyNameDesc());
            similarVO.setSubStrategy(this.getStrategyName(map.getSubStrategy()));
            similarVO.setThirdStrategy(this.getStrategyName(map.getThirdStrategy()));
            resultList.add(similarVO);
        }
        return resultList.stream().filter(e -> e.getSimilar() != null && e.getSimilar() >= Double.parseDouble(params.getThreshold())).collect(Collectors.toList());
    }

    private String getStrategyName(String strategyKey) {
        IStrategy strategy = StrategyHandleUtils.getStrategy(strategyKey);
        if (strategy == null || strategy == NewSubStrategy.OtherStrategy || strategy.getStrategyId() == -1) {
            return "--";
        }
        return strategy.getStrategyNameDesc();
    }

    private Map<String, Map<String, Double>> transfer2TwoDimensionMap(List<Map<String, Object>> initRetList) {
        Map<String, Map<String, Double>> map = new TreeMap<>();
        for (Map<String, Object> unit : initRetList) {
            if (unit != null) {
                String id = unit.get("fund_id").toString();
                if (!map.containsKey(id)) {
                    map.put(id, new TreeMap<>());
                }
                Object t = unit.get("ret_range");
                if (t == null) {
                    continue;
                }
                map.get(id).put(unit.get("end_date").toString(), Double.valueOf(unit.get("ret_range").toString()));
            }

        }
        return map;
    }

    public List<ManualFundNoticeInfoVO> noticeList(ManualFundNoticeReq params) {
        Map<String, String> typeMapper = MapUtil.builder("1", "10").put("2", "20").put("3", "30").put("4", "50, 70, 90, 91").build();
        String fundId = params.getRefId();
        String title = params.getTitle();
        // type 逗号拆分 转list，直接数据库in查询
        List<String> typeList = ListUtil.list(false);
        if (CharSequenceUtil.isNotBlank(params.getType())) {
            String s = typeMapper.get(params.getType());
            typeList.addAll(StrUtil.split(s, ","));
        }
        //todo 确认基金公告表所在的库
        List<ManualFundNoticeInfoDO> dataList = this.baseInfoService.queryFundAnnounce(fundId, typeList, title);
        List<ManualFundNoticeInfoVO> resultList = ListUtil.list(true);
        // key value互换
        Map<String, String> reverse = MapUtil.reverse(typeMapper);
        for (ManualFundNoticeInfoDO temp : dataList) {
            ManualFundNoticeInfoVO vo = BeanUtil.copyProperties(temp, ManualFundNoticeInfoVO.class);
            vo.setType(reverse.getOrDefault(vo.getType(), "4"));
            vo.setDate(DateUtil.formatDate(temp.getDate()));
            resultList.add(vo);
        }
        return resultList;
    }

    public ManualFundInvestInfoVO investInfo(String refId) {
        ManualFundInvestInfoVO fundInvestInfoVO = new ManualFundInvestInfoVO();
        FundArchivesInfoDO fundArchivesInfo = this.baseInfoService.queryFundArchives(refId);
        if(fundArchivesInfo==null){
            return fundInvestInfoVO;
        }
        fundInvestInfoVO.setInvestmentObjective(fundArchivesInfo.getInvestmentObjective());
        fundInvestInfoVO.setStrategyDescription(fundArchivesInfo.getStrategyDescription());
        fundInvestInfoVO.setInvestmentScope(fundArchivesInfo.getInvestmentScope());
        return fundInvestInfoVO;
    }
}
