package com.smppw.analysis.client;

import com.smppw.analysis.application.dto.info.*;
import com.smppw.analysis.application.service.info.FundInfoService;
import com.smppw.analysis.domain.dto.info.FundSimilarVO;
import com.smppw.analysis.domain.dto.info.ManualFundInvestInfoVO;
import com.smppw.analysis.domain.dto.info.ManualFundNoticeInfoVO;
import com.smppw.common.pojo.ResultVo;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author wangzaijun
 * @date 2023/8/7 15:40
 * @description 基本信息api
 */
@RestController
@RequestMapping("/v1/api/fund")
public class FundApi {
    private final FundInfoService service;

    public FundApi(FundInfoService service) {
        this.service = service;
    }

    @GetMapping("head-info")
    public ResultVo<? extends HeadInfoVO> headInfo(HeadInfoReq params) {
        return ResultVo.ok(this.service.headInfo(params));
    }

    @GetMapping("common-info")
    public ResultVo<CommonInfoVO> commonInfo(CommonInfoReq params) {
        return ResultVo.ok(this.service.commonInfo(params));
    }

    @GetMapping("head-indicator")
    public ResultVo<HeadIndicatorVO> headIndicator(HeadIndicatorReq params) {
        return ResultVo.ok(this.service.headIndicator(params));
    }

    @GetMapping("similar")
    public ResultVo<List<FundSimilarVO>> similar(FundSimilarReq params) {
        return ResultVo.ok(this.service.getFundSimilarList(params));
    }

    /**
     * 公募基金公告
     *
     * @param params 基金id
     * @return 基金公告
     */
    @GetMapping("notice-info")
    public ResultVo<List<ManualFundNoticeInfoVO>> noticeInfo(ManualFundNoticeReq params) {
        return ResultVo.ok(service.noticeList(params));
    }

    /**
     * 基金投资类信息说明
     *
     * @param refId 基金id
     * @return 投资类信息说明
     */
    @GetMapping("invest-info")
    public ResultVo<ManualFundInvestInfoVO> investInfo(@RequestParam("refId") String refId) {
        return ResultVo.ok(service.investInfo(refId));
    }

}
